      subroutine vsv_qp(u1,n1,p1,u2,n2,p2,u,n,p,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a polarization vector array being a contraction of a 
c triple vector-scalar boson vertex with a scalar and polarization
c vector
c
c Input: u1 - vector array,
c        u2 - scalar array,
c        n1,n2,n - numbers of polarization states,
c        p1,p2 - momenta,
c        g - a triple scalar-vector boson coupling,
c        mass, wdth - mass and width of u.
c Output: u - vector array,
c         p - its momentum.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 17 May, 2007 (based on vvh.f); modified on 
c 9 March, 2020: quadruple precision version written
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: n1,n2,n,i1,i2,i

      real(dp) :: mass,wdth
      real(qp), dimension(0:3) :: p1,p2,p
      complex(dp) :: g,mv2
      complex(qp) :: u1(0:3,n1),u2(1,n2),u(0:3,n),gden

      mv2=cmplx(mass**2,-mass*wdth,dp)

      p=-p1-p2

      gden=g/(cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2,0.q0,qp)-mv2)

      i=0
      do i2=1,n2
       do i1=1,n1
        i=i+1
        u(:,i)=-gden*u1(:,i1)*u2(1,i2)
        if (mass > 0.d0) then
         u(:,i)=u(:,i)-(p(0)*u(0,i)-p(1)*u(1,i)
     &                 -p(2)*u(2,i)-p(3)*u(3,i))*p/mv2
        end if
       end do
      end do

      end subroutine vsv_qp
