      subroutine fefan(bu,nb,e,ne,p,u,n,ip,gwtb,mat)
c----------------------------------------------------------------------c
c Returns matrix elements with the anomalous Wtb coupling 
c see K. Kolodziej, Phys. Lett. B584 (2004) 89, hep-ph/0312168,
c Input: 
c        bu,u - spinors,
c        e - a polarization vector,
c        p - four momentum of incoming e,
c        nb,ne,n - numbers of polarization states,
c        ip=+1/-1 for u representing a particle/an antiparticle
c        if ip=+1 => Wtb coupling is given by gwtb(1:2,1:2), Eq. (4)
c        if ip=-1 => Wtb coupling is given by gwtb(1:2,3:4), Eq. (5)
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 31 Jan.- 1 Feb., 2011
c----------------------------------------------------------------------c

      implicit none

      integer nb,ne,n,ip

      integer, parameter :: dp=kind(1.d0)

      real(dp) :: p(0:3)
      complex(dp) :: gwtb(2,4)
      complex(dp), dimension(4,nb) :: bu
      complex(dp), dimension(4,n) :: u
      complex(dp), dimension(0:3,ne) :: e
      complex(dp), dimension(nb,ne,n) :: mat

c Local variables:

      integer ib,ie,i

      complex(dp), parameter :: ocom=(0.d0,0.d0),icom=(0.d0,1.d0)
      complex(dp) :: f1l,f1r,f2l,f2r
      complex(dp), dimension(4) :: u1
      complex(dp), dimension(2,2) :: b,c,ae,de
      complex(dp), dimension(0:3,2,2) :: a,d

      if (ip == 1) then
       f1r=gwtb(1,1)
       f1l=gwtb(2,1)
       f2r=gwtb(1,2)
       f2l=gwtb(2,2)
      else if (ip == -1) then
       f1r=gwtb(1,3)
       f1l=gwtb(2,3)
       f2r=gwtb(1,4)
       f2l=gwtb(2,4)
      else
       write(*,*)'fefan.f: ip =',ip,' is not allowed'
       stop
      end if

      a(0,1,:)=(/cmplx(p(3),0.d0,dp),cmplx(p(1),-p(2),dp)/)
      a(0,2,:)=(/cmplx(p(1),p(2),dp),-a(0,1,1)/)
      a(1,1,:)=(/cmplx(0.d0,p(2),dp),cmplx(p(0)-p(3),0.d0,dp)/)
      a(1,2,:)=(/cmplx(p(0)+p(3),0.d0,dp),-a(1,1,1)/)
      a(2,1,:)=(/cmplx(0.d0,-p(1),dp),cmplx(0.d0,-p(0)+p(3),dp)/)
      a(2,2,:)=(/-a(2,1,2),cmplx(0.d0,p(1),dp)/)
      a(3,1,:)=(/cmplx(p(0),0.d0,dp),a(0,1,2)/)
      a(3,2,:)=(/-a(0,2,1),-a(3,1,1)/)
      d(0,:,:)=-a(0,:,:)
      d(1,1,:)=(/a(1,1,1),-a(1,2,1)/)
      d(1,2,:)=(/-a(1,1,2),-a(1,1,1)/)
      d(2,1,:)=(/a(2,1,1),cmplx(0.d0,p(0)+p(3),dp)/)
      d(2,2,:)=(/a(2,1,2),a(2,2,2)/)
      d(3,1,:)=(/a(3,2,2),a(3,1,2)/)
      d(3,2,:)=(/a(3,2,1),a(3,1,1)/)

      do ie=1,ne
       b(1,:)=(/e(0,ie)-e(3,ie),-e(1,ie)+icom*e(2,ie)/)
       b(2,:)=(/-e(1,ie)-icom*e(2,ie),e(0,ie)+e(3,ie)/)
       c(1,:)=(/b(2,2),-b(1,2)/)
       c(2,:)=(/-b(2,1),b(1,1)/)
       ae=a(0,:,:)*e(0,ie)-a(1,:,:)*e(1,ie)
     &   -a(2,:,:)*e(2,ie)-a(3,:,:)*e(3,ie)
       de=d(0,:,:)*e(0,ie)-d(1,:,:)*e(1,ie)
     &   -d(2,:,:)*e(2,ie)-d(3,:,:)*e(3,ie)
       do i=1,n
        u1(:2)=f2l*matmul(ae(:,:),u(:2,i))+f1r*matmul(b(:,:),u(3:,i))
        u1(3:)=f1l*matmul(c(:,:),u(:2,i))+f2r*matmul(de(:,:),u(3:,i))
        do ib=1,nb
         mat(ib,ie,i)=bu(1,ib)*u1(1)+bu(2,ib)*u1(2)
     &               +bu(3,ib)*u1(3)+bu(4,ib)*u1(4)
        end do
       end do
      end do

      end




