      subroutine kinamap_qp(imom,ecm,sqsp,m,c1,c2,x,p,dlips)
c----------------------------------------------------------------------c
c Returns cartezian components of the final state photon four momentum p
c in the c.m.s. with the polar angle ct in [c1,c2] generated according 
c to 1/(1-beta**2*ct**2) and the phase space normalization for a 2 --> 2 
c process. The factor (2*pi)^-2 is not included in dlips.
c
c Input: imom=1/(else) => momentum pa is/(not) calculated
c        ecm - the c.m.s. energy,
c        m - mass of a particle radiating the photon,
c        x(i) - random variables, 0 < x(i) < 1, i=1,2,
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 12 Dec., 2018; modified on 
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0),qp=kind(1.q0)
      integer :: imom
      real(dp) :: c1,c2,x(2),dlips
      real(dp), parameter :: pi=3.141592653589793238d0,!46264338327950q0,
     &                       dphi=2.d0*pi
      real(dp) :: phi
      real(qp) :: ecm,sqsp,m,p(0:3),beta,b1,b2,y,ct,dct,st

      beta=sqrt(1.q0-(2.d0*m/ecm)**2)
      b1=(1.q0+beta*c1)/(1.q0-beta*c1)
      b2=(1.q0+beta*c2)/(1.q0-beta*c2)
      p(0)=(ecm-sqsp)*(ecm+sqsp)/(2.q0*ecm)
      if (imom == 1) then
       y=b1*(b2/b1)**x(1)
       ct=(y-1.q0)/(y+1.q0)/beta
       st=sqrt(1.q0-ct**2)
       phi=dphi*x(2)
       p(1:3)=p(0)*(/st*cos(phi),st*sin(phi),ct/)
      else
       ct=p(3)/p(0)
      end if
      dct=(1.q0-(beta*ct)**2)/(2.q0*beta)*log(b2/b1)
      dlips=p(0)/(4.d0*ecm)*dct*dphi

      end
