      subroutine nuclff_phok(mass,q2,f1,f2)
c----------------------------------------------------------------------c
c Returns nucleon form factors f1 and f2 as in PHOKARA
c Input: 
c        mass - mass and magnetic moment of a nucleon
c        q2 - squared four mometum transfer
c Factor 1/(2*mass) is included in f_2.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 26 Feb., 2014
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      real(dp) :: mass,q2
      complex(dp) :: f1,f2,F1ss,F2ss,F1vv,F2vv
      real(dp) :: ref1,ref2
      real(dp), parameter :: mnucl=0.93892, ! this is only a criterion
     &                       mamom_p = 2.792847356D0,
     &                       mamom_n = -1.9130427D0,
     &                       aaaa =  mamom_p - mamom_n -1.d0,
     &                       bbbb = -mamom_p - mamom_n +1.d0 
      complex(dp), parameter :: 
     & par1  = (-0.12631797530427491d0, -1.4843664898372850d0  ),
     & par2  = (-0.22332499734008593d0, 0.50130368777925582d0  ),
     & par3  = ( 6.22217909274950381d-002, 0.37690806887202166d0),
     & par4  = (  1.4502981859762230d0 ,-0.41455045625322384d0),
     & par5  = ( -1.2962602815823487d0, 0.65876457759446794d0),
     & par6  = ( 2.68791057174262681d-002,-0.23566450991976640d0),
     & par7  = (-0.83158166495785224d0   , -1.8222345444036989d0),
     & par8  = ( 0.14151291995634482d0   ,  1.7216386912021320d0),
     & par9  = (-0.15892167391448200d0   , 0.25682249917576350d0),
     & par10 = (-0.81662751151277746d0   , 0.18008568927744761d0),
     & par11 = ( 0.39918067202581892d0   ,-9.66076873228558231d-002),
     & par12 = ( 6.59809080737624488d-003, 2.24057107274727163d-002)

      real(dp), dimension(0:3) :: mrho_pp,momega_pp,gamrho_pp,
     &                            gamomega_pp
      integer :: frun=1
      save frun,mrho_pp,momega_pp,gamrho_pp,gamomega_pp

      if (frun == 1) then
       mrho_pp(0)     = 0.77549d0
       gamrho_pp(0)   = 0.1491d0
       momega_pp(0)   = 0.78265d0
       gamomega_pp(0) = 0.00849d0
       mrho_pp(1)     = 1.465d0
       gamrho_pp(1)   = 0.4d0
       momega_pp(1)   = 1.425d0
       gamomega_pp(1) = 0.215d0
       mrho_pp(2)     = 1.72d0
       gamrho_pp(2)   = 0.25d0
       momega_pp(2)   = 1.67d0
       gamomega_pp(2) = 0.315d0
       mrho_pp(3)     = 2.12d0           !Phys. Rev. D81 094014 (2010)
       gamrho_pp(3)   = 0.3d0            !Phys. Rev. D81 094014 (2010)
       momega_pp(3)   = 2.0707d0         !Phys. Rev. D81 094014 (2010)
       gamomega_pp(3) = 0.5*momega_pp(3) !Phys. Rev. D81 094014 (2010)
       frun=0
      end if

      F1ss =0.5d0*(bw_fn(momega_pp(0),gamomega_pp(0),q2) 
     &       +par1*bw_fn(momega_pp(1),gamomega_pp(1),q2) 
     &       +par2*bw_fn(momega_pp(2),gamomega_pp(2),q2) 
     &       +par3*bw_fn(momega_pp(3),gamomega_pp(3),q2)) 
     &       /(1.d0+par1+par2+par3)

      F1vv =0.5d0*(bw_fn(mrho_pp(0),gamrho_pp(0),q2) 
     &       +par4*bw_fn(mrho_pp(1),gamrho_pp(1),q2) 
     &       +par5*bw_fn(mrho_pp(2),gamrho_pp(2),q2) 
     &       +par6*bw_fn(mrho_pp(3),gamrho_pp(3),q2))
     &      /(1.d0+par4+par5+par6)
 
      F2ss =-0.5d0*bbbb*(bw_fn(momega_pp(0),gamomega_pp(0),q2) 
     &        +par7*bw_fn(momega_pp(1),gamomega_pp(1),q2) 
     &         +par8*bw_fn(momega_pp(2),gamomega_pp(2),q2) 
     &         +par9*bw_fn(momega_pp(3),gamomega_pp(3),q2)) 
     &         /(1.d0+par7+par8+par9)

      F2vv =0.5d0*aaaa*(bw_fn(mrho_pp(0),gamrho_pp(0),q2) 
     &       +par10*bw_fn(mrho_pp(1),gamrho_pp(1),q2) 
     &       +par11*bw_fn(mrho_pp(2),gamrho_pp(2),q2) 
     &       +par12*bw_fn(mrho_pp(3),gamrho_pp(3),q2))
     &        /(1.d0+par10+par11+par12)

      if (mass < mnucl) then ! proton
       f1 = F1ss+F1vv
       f2 = (F2ss+F2vv)/(2.d0*mass)
      else ! neutron
       f1 = F1ss-F1vv
       f2 = (F2ss-F2vv)/(2.d0*mass)
      end if

      if (q2 <= 0.d0) then ! ignore the imaginary part in the spacelike region
       ref1=f1
       ref2=f2
       f1=cmplx(ref1,0.d0,dp)
       f2=cmplx(ref2,0.d0,dp)
      end if

      contains 

      function bw_fn(m,wdth,x)
      implicit none
      integer, parameter :: dp=kind(1.d0)
      complex(dp) :: bw_fn
      real(dp) :: m,wdth,x
      complex(dp), parameter :: i=(0.d0,1.d0)

      if(x > 0.d0)then
       bw_fn=cmplx(m*m*(m*m-x)/((m*m-x)**2+(m*wdth)**2),m*m*m*wdth
     &      /((m*m-x)**2+(m*wdth)**2),dp)
      else
       bw_fn=cmplx(m*m/(m*m-x),0.d0,dp)
      endif
      end function bw_fn

      end
