      subroutine sss_qp(u1,n1,p1,u2,n2,p2,u,n,p,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a scalar array being a contraction of a triple scalar boson 
c vertex with two scalars
c
c Input: u1,u2 - sclars,
c        n1,n2,n - numbers of polarization states,
c        p1,p2 - momenta,
c        g - a triple scalar boson coupling,
c        mass,wdth - mass and width of u.
c Output: u - scalar,
c         p - momentum of u.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 16 May, 2007 (based on hhh.f); modified on 
c 9 March, 2020 (quadruple precision version written)
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: i1,i2,i,n1,n2,n

      real(dp) :: mass,wdth
      real(qp), dimension(0:3) :: p1,p2,p
      complex(dp) :: g
      complex(qp) :: u1(1,n1),u2(1,n2),u(1,n),gden

      p=-p1-p2

      gden=g/cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2-mass**2,
     &                                                 mass*wdth,qp)

      i=0

      do i2=1,n2
       do i1=1,n1
        i=i+1
        u(1,i)=gden*u1(1,i1)*u2(1,i2)
       end do
      end do

      end subroutine sss_qp
