      subroutine svsvmd_qp(ig,g,u1,p1,u2,p2,u3,p3,u4,mat)
c----------------------------------------------------------------------c
c Returns a matrix element array for a quartic gauge-Higgs boson vertex 
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        p1,p2,p3 - four momenta,
c        u2,u4 - vector,
c        u1,u3 - sclar,
c        g - a quartic scalar-gauge-boson coupling.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 3 June, 2007; modified 
c 1 April, 2105:
c a call to s-dependent coupling implemented
c 5 March, 2020: 
c quadruple precision version written
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: ig
      real(qp), dimension(0:3) :: p1,p2,p3,p
      complex(dp) :: g,rg
      complex(qp) :: u2(0:3),u4(0:3),u1(1),u3(1),mat

      p=-p1-p2-p3

      if (ig == 0) then
       rg=g
      else
       call runcoupl_qp(ig,p,rg)
      end if

      mat=-rg*u1(1)*u3(1)*(u2(0)*u4(0)-u2(1)*u4(1)
     &                    -u2(2)*u4(2)-u2(3)*u4(3))

c The extra minus sign results from missing i^2 if a quartic coupling 
c is present

      end subroutine svsvmd_qp
