      subroutine vvpmd(ig,g,u1,p1,u2,p2,u,mat)
c----------------------------------------------------------------------c
c Returns a matrix element for a triple pi0-vector meson-vector meson 
c coupling contracted with vectors u1 and u2 and a scalar u
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        g - a triple pi0-vector meson-vector meson coupling,
c        u1,u2 - vectors,
c        u - scalar,
c        p1,p2 - momenta.
c        
c Output: mat - the matrix element.
c All four momenta are considered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 25 Sept., 2014; modified on 
c 19 March, 2015 (call to subroutine runcoupl implemented)
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: ig
      real(dp), dimension(0:3) :: p1,p2,p
      complex(dp) :: u1(0:3),u2(0:3),u(1),g,rg,mat

      p=-p1-p2

      if (ig == 0) then
       rg=g
      else
       call runcoupl(ig,p,rg)
      end if

      mat=rg*u(1)*(
     &      u1(0)*(-u2(1)*cmplx(p1(2)*p2(3)-p1(3)*p2(2),0.d0,dp)
     &             +u2(2)*cmplx(p1(1)*p2(3)-p1(3)*p2(1),0.d0,dp)
     &             -u2(3)*cmplx(p1(1)*p2(2)-p1(2)*p2(1),0.d0,dp))
     &     -u1(1)*(-u2(0)*cmplx(p1(2)*p2(3)-p1(3)*p2(2),0.d0,dp)
     &             +u2(2)*cmplx(p1(0)*p2(3)-p1(3)*p2(0),0.d0,dp)
     &             -u2(3)*cmplx(p1(0)*p2(2)-p1(2)*p2(0),0.d0,dp))
     &     -u1(2)*( u2(0)*cmplx(p1(1)*p2(3)-p1(3)*p2(1),0.d0,dp)
     &             -u2(1)*cmplx(p1(0)*p2(3)-p1(3)*p2(0),0.d0,dp)
     &             +u2(3)*cmplx(p1(0)*p2(1)-p1(1)*p2(0),0.d0,dp))
     &     -u1(3)*(-u2(0)*cmplx(p1(1)*p2(2)-p1(2)*p2(1),0.d0,dp)
     &             +u2(1)*cmplx(p1(0)*p2(2)-p1(2)*p2(0),0.d0,dp)
     &             -u2(2)*cmplx(p1(0)*p2(1)-p1(1)*p2(0),0.d0,dp)))

      end subroutine vvpmd
