      subroutine wtbmd_qp(pe,gwtb,e,bu,u,mat)
c----------------------------------------------------------------------c
c Returns the matrix element u*e*gwtb*bu with the anomalous Wtb coupling 
c
c Input: 
c        pe - four momentum of incoming e,
c        gwtb - the coupling given by Eq. (4) of K. Kolodziej, Phys. 
c               Lett. B584 (2004) 89, hep-ph/0312168,
c        e -  polarization vector of W
c        bu - spinor of the t~-quark,
c        u - spinor of the b~-quark
c
c The Dirac gamma matrices are in the Weyl representation
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 10 Feb., 2011
c 11 April, 2011: error in a(2,2,1) corrected
c 9 March, 2020 quadruple precision version written
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)

      real(qp), dimension(0:3) :: pe
      complex(dp) :: gwtb(2,4)
      complex(qp) :: mat
      complex(qp), dimension(0:3) :: e
      complex(qp), dimension(4) :: bu,u

c Local variables:

      complex(qp), parameter :: icom=(0.q0,1.q0)
      complex(dp) :: f1l,f1r,f2l,f2r
      complex(qp), dimension(4) :: u1
      complex(qp), dimension(2,2) :: b,c,ae,de
      complex(qp), dimension(0:3,2,2) :: a,d


      f1r=gwtb(1,1)
      f1l=gwtb(2,1)
      f2r=gwtb(1,2)
      f2l=gwtb(2,2)

      a(0,1,:)=(/cmplx(pe(3),0.q0,qp),cmplx(pe(1),-pe(2),qp)/)
      a(0,2,:)=(/cmplx(pe(1),pe(2),qp),-a(0,1,1)/)
      a(1,1,:)=(/cmplx(0.q0,pe(2),qp),cmplx(pe(0)-pe(3),0.q0,qp)/)
      a(1,2,:)=(/cmplx(pe(0)+pe(3),0.q0,qp),-a(1,1,1)/)
      a(2,1,:)=(/cmplx(0.q0,-pe(1),qp),cmplx(0.q0,-pe(0)+pe(3),qp)/)
      a(2,2,:)=(/cmplx(0.q0,pe(0)+pe(3),qp),-a(2,1,1)/)
      a(3,1,:)=(/cmplx(pe(0),0.q0,qp),a(0,1,2)/)
      a(3,2,:)=(/-a(0,2,1),-a(3,1,1)/)
      d(0,:,:)=-a(0,:,:)
      d(1,1,:)=(/a(1,1,1),-a(1,2,1)/)
      d(1,2,:)=(/-a(1,1,2),-a(1,1,1)/)
      d(2,1,:)=(/a(2,1,1),a(2,2,1)/)
      d(2,2,:)=(/a(2,1,2),a(2,2,2)/)
      d(3,1,:)=(/a(3,2,2),a(3,1,2)/)
      d(3,2,:)=(/a(3,2,1),a(3,1,1)/)

      b(1,:)=(/e(0)-e(3),-e(1)+icom*e(2)/)
      b(2,:)=(/-e(1)-icom*e(2),e(0)+e(3)/)
      c(1,:)=(/b(2,2),-b(1,2)/)
      c(2,:)=(/-b(2,1),b(1,1)/)
      ae=a(0,:,:)*e(0)-a(1,:,:)*e(1)-a(2,:,:)*e(2)-a(3,:,:)*e(3)
      de=d(0,:,:)*e(0)-d(1,:,:)*e(1)-d(2,:,:)*e(2)-d(3,:,:)*e(3)
      u1(:2)=f2l*matmul(ae,bu(:2))+f1r*matmul(b,bu(3:))
      u1(3:)=f1l*matmul(c,bu(:2))+f2r*matmul(de,bu(3:))

      mat=u1(1)*u(1)+u1(2)*u(2)+u1(3)*u(3)+u1(4)*u(4)

      end




